Package: spatialwarnings
Type: Package
Title: Spatial Early Warning Signals of Ecosystem Degradation
Version: 3.1.2
Authors@R: c(person(given = "Alain",
                    family = "Danet",
                    role = "aut"),
             person(given = "Alexandre",
                    family = "Genin",
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-3333-1338"), 
                    email = "alexandre.genin@inrae.fr"),
             person(given = "Vishwesha",
                    family = "Guttal",
                    role = "aut"),
             person(given = "Sonia",
                    family = "Kefi",
                    role = "aut"),
             person(given = "Sabiha",
                    family = "Majumder",
                    role = "aut"),
             person(given = "Sumithra",
                    family = "Sankaran",
                    role = "aut"),
             person(given = "Florian",
                    family = "Schneider",
                    role = "aut"))
License: MIT + file LICENSE
Maintainer: Alexandre Genin <alexandre.genin@inrae.fr>
Description: Tools to compute and assess significance of early-warnings signals (EWS) of ecosystem degradation. EWS are spatial metrics derived from raster data -- e.g. spatial autocorrelation -- that increase before an ecosystem undergoes a non-linear transition (Genin et al. (2018) <doi:10.1111/2041-210X.13058>).
URL: https://github.com/spatial-ews/spatialwarnings
BugReports: https://github.com/spatial-ews/spatialwarnings/issues
Depends: R (>= 3.5.0), future
Imports: Rcpp, ggplot2, plyr, stats, utils, future.apply, gsl,
        segmented
Suggests: moments, poweRlaw, reshape2, testthat, covr, acss, acss.data,
        mgcv, gstat, sp, raster
LinkingTo: Rcpp, RcppArmadillo
LazyData: true
RoxygenNote: 7.3.2.9000
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-07-23 10:55:16 UTC; alex
Author: Alain Danet [aut],
  Alexandre Genin [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3333-1338>),
  Vishwesha Guttal [aut],
  Sonia Kefi [aut],
  Sabiha Majumder [aut],
  Sumithra Sankaran [aut],
  Florian Schneider [aut]
Repository: CRAN
Date/Publication: 2026-07-23 15:50:02 UTC
