# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,patchdistr_sews_list)
S3method(as.data.frame,patchdistr_sews_single)
S3method(as.data.frame,patchdistr_sews_test_list)
S3method(as.data.frame,patchdistr_sews_test_single)
S3method(as.data.frame,simple_sews_list)
S3method(as.data.frame,simple_sews_single)
S3method(as.data.frame,simple_sews_test_list)
S3method(as.data.frame,simple_sews_test_single)
S3method(convert_to_matrix,factor)
S3method(convert_to_matrix,list)
S3method(convert_to_matrix,matrix)
S3method(display_matrix,default)
S3method(display_matrix,list)
S3method(display_matrix,matrix)
S3method(display_matrix,sews_result_list)
S3method(display_matrix,sews_result_single)
S3method(display_size_info,list)
S3method(display_size_info,sews_result_list)
S3method(display_size_info,sews_result_single)
S3method(extract_spectrum,spectral_sews_list)
S3method(extract_spectrum,spectral_sews_single)
S3method(extract_variogram,variogram_sews_list)
S3method(extract_variogram,variogram_sews_single)
S3method(indictest,flowlength_sews_list)
S3method(indictest,flowlength_sews_single)
S3method(indictest,patchdistr_sews_list)
S3method(indictest,patchdistr_sews_single)
S3method(indictest,simple_sews)
S3method(indictest,simple_sews_list)
S3method(indictest,simple_sews_single)
S3method(indictest,spectral_sews_list)
S3method(indictest,spectral_sews_single)
S3method(indictest,variogram_sews_list)
S3method(indictest,variogram_sews_single)
S3method(plot,patchdistr_sews)
S3method(plot,patchdistr_sews_list)
S3method(plot,simple_sews_list)
S3method(plot,simple_sews_test)
S3method(plot,simple_sews_test_list)
S3method(plot,variogram_sews)
S3method(plot,variogram_sews_list)
S3method(plot,variogram_sews_test)
S3method(plot,variogram_sews_test_list)
S3method(plot_distr,patchdistr_sews)
S3method(plot_distr,patchdistr_sews_list)
S3method(plot_distr,patchdistr_sews_single)
S3method(plot_distr,patchdistr_sews_test_list)
S3method(plot_distr,patchdistr_sews_test_single)
S3method(plot_spectrum,spectral_sews_list)
S3method(plot_spectrum,spectral_sews_single)
S3method(plot_spectrum,spectral_sews_test_list)
S3method(plot_spectrum,spectral_sews_test_single)
S3method(plot_variogram,variogram_sews)
S3method(plot_variogram,variogram_sews_list)
S3method(plot_variogram,variogram_sews_single)
S3method(plot_variogram,variogram_sews_test)
S3method(plot_variogram,variogram_sews_test_list)
S3method(plot_variogram,variogram_sews_test_single)
S3method(predict,patchdistr_sews_list)
S3method(predict,patchdistr_sews_single)
S3method(predict,variogram_sews_list)
S3method(predict,variogram_sews_single)
S3method(print,patchdistr_sews_list)
S3method(print,patchdistr_sews_single)
S3method(print,patchdistr_sews_test_list)
S3method(print,patchdistr_sews_test_single)
S3method(print,simple_sews_list)
S3method(print,simple_sews_single)
S3method(print,simple_sews_test_list)
S3method(print,simple_sews_test_single)
S3method(summary,patchdistr_sews_list)
S3method(summary,patchdistr_sews_single)
S3method(summary,patchdistr_sews_test_list)
S3method(summary,patchdistr_sews_test_single)
S3method(summary,simple_sews_list)
S3method(summary,simple_sews_single)
S3method(summary,simple_sews_test_list)
S3method(summary,simple_sews_test_single)
export(LSW_fit)
export(coarse_grain)
export(compute_indicator)
export(convert_to_matrix)
export(create_indicator)
export(dLSW)
export(display_matrix)
export(display_size_info)
export(exp_fit)
export(extract_spectrum)
export(extract_variogram)
export(flowlength_sews)
export(generic_sews)
export(indicator_plrange)
export(indicator_psdtype)
export(indictest)
export(kbdm_sews)
export(label)
export(lnorm_fit)
export(lsw_sews)
export(pLSW)
export(pair_counts)
export(patchdistr_sews)
export(patchsizes)
export(percolation)
export(pl_fit)
export(plot_distr)
export(plot_spectrum)
export(plot_variogram)
export(raw_cg_moran)
export(raw_cg_skewness)
export(raw_cg_variance)
export(raw_clustering)
export(raw_flowlength_uniform)
export(raw_kbdm)
export(raw_lsw_aicw)
export(raw_moran)
export(raw_patch_radii_skewness)
export(raw_plrange)
export(raw_sdr)
export(raw_structvar)
export(raw_variogram_metrics)
export(rspectrum)
export(spectral_sews)
export(tpl_fit)
export(variogram_sews)
export(xmin_estim)
import(ggplot2)
import(plyr)
import(stats)
import(utils)
importFrom(Rcpp,sourceCpp)
importFrom(future,plan)
useDynLib(spatialwarnings)
